--
-- Created by IntelliJ IDEA.
-- User: insai
-- Date: 7/7/2016
-- Time: 9:55 PM
-- To change this template use File | Settings | File Templates.
--

KindredUtils = LibStub("AceAddon-3.0"):NewAddon("KindredUtils", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceComm-3.0", "AceSerializer-3.0")

KUTIL_ColorIndex = {0,10,15,20,25,30,35}
KUTIL_Colors = {
    -- White base color
    [0] =       {['r']=255,     ['g']=255,      ['b']=255,      ['rgb']='FFFFFF',   ['p']=0,},
    -- Yellow for > 10 traits
    [10] =     {['r']=255,     ['g']=255,      ['b']=0,        ['rgb']='FFFF00',   ['p']=0,},
    -- Green for > 15 traits
    [15] =     {['r']=0,       ['g']=255,      ['b']=0,        ['rgb']='00FF00',   ['p']=10,},
    -- Teal for > 20 traits
    [20] =     {['r']=0,       ['g']=255,      ['b']=255,      ['rgb']='00FFFF',   ['p']=15,},
    -- Blue for > 25 traits
    [25] =     {['r']=0,       ['g']=102,      ['b']=255,      ['rgb']='0066ff',   ['p']=20,},
    -- Purple for > 30 traits
    [30] =     {['r']=255,     ['g']=0,        ['b']=255,      ['rgb']='FF00FF',   ['p']=25,},
    -- Red for >= 35 traits
    [35] =    {['r']=255,     ['g']=0,        ['b']=0,        ['rgb']='FF0000',   ['p']=30,},
}
KUTIL_grayScore = 9
KUTIL_prefix = "KUTIL_"
KUTIL_prefixArtifacts = "artifacts"

function KindredUtils:TryUpdateArtifact()
    --pcall(self:UpdateArtifact())
    self:UpdateArtifact()
end

function KindredUtils:UpdateArtifact()
    -- Set our own characters GUID as a global to avoid lookups in the future
    if self.MyUnitGUID == nil then
        self.MyUnitGUID = self:NameToGUID(UnitName("player"))
    end
    -- If we still can't get our own GUID, end this loop
    if self.MyUnitGUID == nil then return end
    local var1, var2, artifactName, var3, var4, artifactTraits, var5, var6, vary, var9, var10 = C_ArtifactUI:GetEquippedArtifactInfo()
    local myArtifacts = self.db.global.artifactCache[self.MyUnitGUID]
    if artifactName and artifactTraits then
        if myArtifacts == nil then
            myArtifacts = {}
        end
        myArtifacts[artifactName] = artifactTraits
        self.db.global.artifactCache[self.MyUnitGUID] = myArtifacts
    end
    self:BroadcastArtifacts()   
end

function KindredUtils:DebugPrint()
    local name = UnitName("player")
    local guid = UnitGUID(name)
    self:Print("GUID: " .. guid)
    for k,v in pairs(self.db.global.artifactCache[UnitGUID("player")]) do
        self:Print("Your artifact [" .. k .. "] has " .. v .. " traits unlocked.")
    end
    self:PrintR(self.db.global.artifactCache)
    self:BroadcastArtifacts()
end

--
-- Credit where credit is due - The following functions were borrowed from SimpleILevel by Scotepi
--

-- Used to hook the tooltip to avoid the full tooltip function
function KindredUtils:TooltipHook()
    local name, unit = GameTooltip:GetUnit();
    local guid = false;
    
    if unit then
        guid = UnitGUID(unit);
    elseif name then
        guid = self:NameToGUID(name);
    end
    
    if self:IsGUID(guid, 'Player') then
        self:ShowTooltip(guid);
    end
end

function KindredUtils:NameToGUID(name, realm)
    if name then
        return UnitGUID(name)
    end
    return false;
end

function KindredUtils:IsGUID(guid, type)
    if not guid then return false end
    if not type then type = 'player' end

    local gType, gGuid = strsplit('-', guid, 2);

    if strlower(type) == strlower(gType) then 
        return true
    else 
        return false
    end
end

function KindredUtils:ShowTooltip(guid)
    if InCombatLockdown() then return end
    
    if not guid then
        guid = UnitGUID("mouseover");
    end
    
    local weapons = self:GetArtifactWeapons(guid);
    
    if weapons then
        for k,v in pairs(weapons) do
            -- Build the tooltip text
            local traitsHex, r, g, b = self:ColorArtifactTraits(v)
            local textLeft = '|cFF216bff' .. k .. ':|r '
            local textRight = '|cFF' .. traitsHex .. ' ' .. v .. ' traits unlocked|r'
            self:AddTooltipText(textLeft, textRight);
        end
    else
        local textLeft = '|cFF216bff' .. 'No Artifact Information Available' .. ':|r '
        self:AddTooltipText(textLeft, '');
    end
    return true;
end

function KindredUtils:GetArtifactWeapons(guid)
    if not self:IsGUID(guid, 'Player') then return false; end

    if self.db.global.artifactCache[guid] then
        return self.db.global.artifactCache[guid]
    else
        return {}
    end
end

-- Return the hex, r, g, b of a score
function KindredUtils:ColorArtifactTraits(traits)
    -- There are some missing items so gray
    if traits and traits <= KUTIL_grayScore then
        return self:RGBtoHex(0.5,0.5,0.5), 0.5,0.5,0.5;
    end
    
    -- Default to white
    local r,g,b = 1,1,1;
    
    local found = false;

    for i,maxScore in pairs(KUTIL_ColorIndex) do
        if traits < maxScore and not found then
            local colors = KUTIL_Colors[maxScore];
            local baseColors = KUTIL_Colors[colors.p];
            
            local steps = maxScore - colors.p;
            local scoreDiff = traits - colors.p;
            
            local diffR = (baseColors.r - colors.r) / 255;
            local diffG = (baseColors.g - colors.g) / 255;
            local diffB = (baseColors.b - colors.b) / 255;
            
            local diffStepR = diffR / steps;
            local diffStepG = diffG / steps;
            local diffStepB = diffB / steps;
            
            local scoreDiffR = scoreDiff * diffStepR;
            local scoreDiffG = scoreDiff * diffStepG;
            local scoreDiffB = scoreDiff * diffStepB;
            
            r = (baseColors.r / 255) - scoreDiffR;
            g = (baseColors.g / 255) - scoreDiffG;
            b = (baseColors.b / 255) - scoreDiffB;
            
            found = true;
        end
    end
    
    -- Nothing was found so max
    if not found then
        r = KUTIL_Colors[35].r;
        g = KUTIL_Colors[35].g;
        b = KUTIL_Colors[35].b;
    end
    
    local hex = self:RGBtoHex(r,g,b);
    return hex, r, g, b;
end

-- from http://www.wowpedia.org/RGBPercToHex
function KindredUtils:RGBtoHex(r, g, b)
    r = r <= 1 and r >= 0 and r or 0
    g = g <= 1 and g >= 0 and g or 0
    b = b <= 1 and b >= 0 and b or 0
    return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

-- Add lines to the tooltip, testLeft must be the same
function KindredUtils:AddTooltipText(textLeft, textRight)
    
    -- Loop tooltip text to check if its alredy there
    local ttLines = GameTooltip:NumLines();
    local ttUpdated = false;
    
    for i = 1,ttLines do
        
        -- If the static text matches
        if _G["GameTooltipTextLeft"..i]:GetText() == textLeft then
            
            -- Update the text
            _G["GameTooltipTextLeft"..i]:SetText(textLeft);
            _G["GameTooltipTextRight"..i]:SetText(textRight);
            GameTooltip:Show();
            
            -- Remember that we have updated the tool tip so we won't again
            ttUpdated = true;
            break;
        end
    end
    
    -- Tooltip is new
    if not ttUpdated then
        GameTooltip:AddDoubleLine(textLeft, textRight);
        GameTooltip:Show();
    end
end

--
-- End Credited Code
--

function KindredUtils:SerializeArtifacts()
    if self.MyUnitGUID == nil then return end
    local ret = {}
    ret['GUID'] = self.MyUnitGUID
    ret['artifacts'] = self.db.global.artifactCache[self.MyUnitGUID]
    return self:Serialize(ret)
end

function KindredUtils:BroadcastArtifacts()
    -- Be kind, send low-priority messages, only send one every 5 seconds to avoid spamming
    if time() - self.lastBroadcast > 5 then
        local message = self:SerializeArtifacts()
        self:SendCommMessage(KUTIL_prefix .. KUTIL_prefixArtifacts, message, "RAID", nil, "BULK")
    end
end

function KindredUtils:OnCommReceived(prefix, message, distribution, sender)
    if prefix == KUTIL_prefix .. KUTIL_prefixArtifacts then
        local success, payload = self:Deserialize(message)
        if success and payload.GUID then
            local thisPersonsArtifacts = {}
            if self.db.global.artifactCache[payload.GUID] then
                thisPersonsArtifacts = self.db.global.artifactCache[payload.GUID]
            end
            if payload.artifacts then
                for k,v in pairs(payload.artifacts) do
                    thisPersonsArtifacts[k] = v
                end
            else
                self:Print("Bad payload received. No Artifacts in message")
            end
            self.db.global.artifactCache[payload.GUID] = thisPersonsArtifacts
        else
            self:Print("Bad payload received. No Character GUID in message")
        end
    end
end

function KindredUtils:PrintR ( t )
    local print_r_cache={}
    local function sub_print_r(t,indent)
        if (print_r_cache[tostring(t)]) then
            print(indent.."*"..tostring(t))
        else
            print_r_cache[tostring(t)]=true
            if (type(t)=="table") then
                for pos,val in pairs(t) do
                    if (type(val)=="table") then
                        print(indent.."["..pos.."] => "..tostring(t).." {")
                        sub_print_r(val,indent..string.rep(" ",string.len(pos)+8))
                        print(indent..string.rep(" ",string.len(pos)+6).."}")
                    elseif (type(val)=="string") then
                        print(indent.."["..pos..'] => "'..val..'"')
                    else
                        print(indent.."["..pos.."] => "..tostring(val))
                    end
                end
            else
                print(indent..tostring(t))
            end
        end
    end
    if (type(t)=="table") then
        print(tostring(t).." {")
        sub_print_r(t,"  ")
        print("}")
    else
        sub_print_r(t,"  ")
    end
    print()
end

function KindredUtils:OnInitialize()
    -- Code that runs when the addon is first loaded
    GameTooltip:HookScript("OnTooltipSetUnit", function(...) KindredUtils:TooltipHook(...); end);
    self.db = LibStub("AceDB-3.0"):New("KindredUtilsDB")
    self:RegisterComm(KUTIL_prefix .. KUTIL_prefixArtifacts)
end

function KindredUtils:OnEnable()
    -- Code that runs when the addon is enabled
    if self.updateTimer == nil then
        self.updateTimer = self:ScheduleRepeatingTimer("TryUpdateArtifact", 5)
    end
    if self.db.global.artifactCache == nil then
        self.db.global.artifactCache = {}
    end 
    self.grayScore = 9
    self:RegisterChatCommand("kindredDebug", "DebugPrint")
    self:RegisterEvent("PARTY_MEMBERS_CHANGED", "BroadcastArtifacts")
    self:RegisterEvent("RAID_ROSTER_UPDATE", "BroadcastArtifacts")
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", "TryUpdateArtifact")
    self.lastBroadcast = time()
    self:TryUpdateArtifact()
end

function KindredUtils:OnDisable()
    -- Code that runs when the addon is disabled
    if self.updateTimer ~= nil then
        self:CancelTimer(self.updateTimer)
    end
    self.db.global.artifactCache = nil
    self:UnregisterChatCommand("kindredDebug")
    self:UnregisterEvent("PARTY_MEMBERS_CHANGED", "BroadcastArtifacts")
    self:UnregisterEvent("RAID_ROSTER_UPDATE", "BroadcastArtifacts")
    self:UnregisterEvent("UNIT_INVENTORY_CHANGED")
end
 









